@@ -66,7 +66,8 @@ update_hints (TaskItem *item)
 {
   GtkWidget *parent;
   GtkWidget *widget;
-  GtkAllocation *allocation;
+  GtkAllocation allocation_parent;
+  GtkAllocation allocation_widget;
   WnckWindow *window;
   GdkWindow *gdkwindow;
   gint x, y, x1, y1;
@@ -87,9 +88,9 @@ update_hints (TaskItem *item)
   {
     if (gtk_widget_get_parent (parent))
     {
-      gtk_widget_get_allocation (parent, allocation);
-      x += allocation->x;
-      y += allocation->y;
+      gtk_widget_get_allocation (parent, &allocation_parent);
+      x += allocation_parent.x;
+      y += allocation_parent.y;
     }
     else
     {
@@ -103,14 +104,12 @@ update_hints (TaskItem *item)
       break;
     }
   }
-    
+
   /* Set the minimize hint for the window */
-  gtk_widget_get_allocation (widget, allocation);
-  if (allocation) {	
-  	wnck_window_set_icon_geometry (window, x, y,
-                                 allocation->width,
-                                 allocation->height);
-  }
+  gtk_widget_get_allocation (widget, &allocation_widget);
+  wnck_window_set_icon_geometry (window, x, y,
+                                 allocation_widget.width,
+                                 allocation_widget.height);
 }
 
 static gboolean 
@@ -138,7 +137,7 @@ on_task_item_button_released (GtkWidget      *widget,
   
     if (WNCK_IS_WORKSPACE (workspace) && workspace != wnck_screen_get_active_workspace (screen))
     {
-      wnck_workspace_activate (workspace, GDK_CURRENT_TIME);
+      wnck_workspace_activate (workspace, event->time);
     }
     if (wnck_window_is_active (window))
     {
@@ -146,7 +145,7 @@ on_task_item_button_released (GtkWidget      *widget,
     }
     else
     {
-      wnck_window_activate (window, GDK_CURRENT_TIME);
+      wnck_window_activate (window, event->time);
     }
   }
   return TRUE;
@@ -292,7 +291,6 @@ task_item_expose_event (GtkWidget      *widget,
   TaskItem *item;
   GdkRectangle area;
   TaskItemPrivate *priv;
-  GdkPixbuf *desat;
   GdkPixbuf *pbuf;
   
   g_return_val_if_fail (widget != NULL, FALSE);
@@ -312,7 +310,6 @@ task_item_expose_event (GtkWidget      *widget,
 #endif
   
   pbuf = priv->pixbuf;
-  desat = NULL;
   
   gint size = MIN (area.height, area.width);
   gboolean active = wnck_window_is_active (priv->window);
@@ -328,13 +325,35 @@ task_item_expose_event (GtkWidget      *widget,
   
   if (active)
   {
-    cairo_rectangle (cr, area.x + .5, area.y - 4, area.width - 1, area.height + 8);
+    cairo_rectangle (cr, area.x + 1, area.y - 1, area.width - 2, area.height - 2);
     cairo_set_source_rgba (cr, .8, .8, .8, .2);
     cairo_fill_preserve (cr);
-    
-    cairo_set_line_width (cr, 1);
-    cairo_set_source_rgba (cr, .8, .8, .8, .4);
-    cairo_stroke (cr);
+    if (priv->mouse_over)
+    {
+      cairo_set_source_rgba (cr, .9, .9, 1, 0.45);
+      cairo_stroke (cr);
+    }
+    else
+    {
+      cairo_set_line_width (cr, 1);
+      cairo_set_source_rgba (cr, .8, .8, .8, .4);
+      cairo_stroke (cr);
+    }
+  }
+  else if (priv->mouse_over)
+  {
+    int glow_x, glow_y;
+    cairo_pattern_t *glow_pattern;
+    glow_x = area.width / 2;
+    glow_y = area.height / 2;
+    glow_pattern = cairo_pattern_create_radial (
+        area.x + glow_x, area.y + glow_y, glow_x * 0.3,
+        area.x + glow_x, area.y + glow_y, glow_x * 1.4
+    );
+    cairo_pattern_add_color_stop_rgba (glow_pattern, 0, 1, 1, 1, 1);
+    cairo_pattern_add_color_stop_rgba (glow_pattern, 0.6, 1, 1, 1, 0);
+    cairo_set_source (cr, glow_pattern);
+    cairo_paint (cr);
   }
   
   if (!pbuf)
@@ -351,7 +370,7 @@ task_item_expose_event (GtkWidget      *widget,
   }
   else
   {
-    desat = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+    GdkPixbuf *desat = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
                             TRUE,
                             gdk_pixbuf_get_bits_per_sample (pbuf),
                             gdk_pixbuf_get_width (pbuf),
@@ -362,16 +381,17 @@ task_item_expose_event (GtkWidget      *widget,
       gdk_pixbuf_saturate_and_pixelate (pbuf,
                                         desat,
                                         0,
-                                       FALSE);
+                                        FALSE);
     }
     else /* just paint the colored version as a fallback */
     {
-      desat = pbuf;
+      desat = g_object_ref (pbuf);
     }
     gdk_cairo_set_source_pixbuf (cr, 
                                  desat, 
                                  (area.x + (area.width - gdk_pixbuf_get_width (desat)) / 2), 
                                  (area.y + (area.height - gdk_pixbuf_get_height (desat)) / 2));
+    g_object_unref (desat);
   }
   if (!priv->mouse_over && attention) /* urgent */
   {
@@ -392,10 +412,7 @@ task_item_expose_event (GtkWidget      *widget,
   {
     cairo_paint_with_alpha (cr, .65);
   }
-  
-  if (GDK_IS_PIXBUF (desat))
-    g_object_unref (desat);
- 
+
 #if !GTK_CHECK_VERSION (3, 0, 0)
   cairo_destroy (cr);
 #endif
