@@ -24,8 +24,6 @@
 #include <mate-panel-applet.h>
 
 #include <math.h>
-#define MATE_DESKTOP_USE_UNSTABLE_API
-#include <libmate-desktop/mate-desktop-utils.h>
 
 #include "task-list.h"
 
@@ -71,17 +69,12 @@ on_close_clicked (GtkButton *button,
 
   window = wnck_screen_get_active_window (priv->screen);
 
-  if (!WNCK_IS_WINDOW (window)
-        || wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP)
-  {
-    mate_gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-                                           LOGOUT, NULL);
-  }
-  else
+  if (WNCK_IS_WINDOW (window)
+        && wnck_window_get_window_type (window) != WNCK_WINDOW_DESKTOP)
   {
     if (priv->window == window)
       disconnect_window (title);
-    wnck_window_close (window, GDK_CURRENT_TIME);
+    wnck_window_close (window, event->time);
   }
   gtk_widget_queue_draw (GTK_WIDGET (title));
 
@@ -114,15 +107,11 @@ on_leave_notify (GtkWidget *widget,
   return FALSE;
 }
 
-static gboolean
 #if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean
 on_button_draw (GtkWidget *widget, 
                 cairo_t *cr,
-#else
-on_button_expose (GtkWidget *widget, 
-                  GdkEventExpose *event,
-#endif
-                  TaskTitle *title)
+                TaskTitle *title)
 {
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
 
@@ -132,7 +121,6 @@ on_button_expose (GtkWidget *widget,
   if (priv->mouse_in_close_button)
   {
     GtkStyle *style = gtk_widget_get_style (widget);
-#if GTK_CHECK_VERSION (3, 0, 0)
     GdkRectangle area;
     gdouble x1, y1, x2, y2;
     cairo_clip_extents (cr, &x1, &y1, &x2, &y2);
@@ -140,34 +128,50 @@ on_button_expose (GtkWidget *widget,
     area.y = floor (y1);
     area.width = ceil (x2) - area.x;
     area.height = ceil (y2) - area.y;
-#endif
+
     gtk_paint_box (style,
-#if GTK_CHECK_VERSION (3, 0, 0)
                    cr,
-#else
-                   event->window,
-#endif
                    GTK_STATE_PRELIGHT,
                    GTK_SHADOW_NONE,
-#if !GTK_CHECK_VERSION (3, 0, 0)
-                   NULL,
-#endif
                    NULL,
                    NULL,
-#if GTK_CHECK_VERSION (3, 0, 0)
                    area.x,
                    area.y + 2,
                    area.width,
                    area.height - 4);
+  }
+  return FALSE;
+}
 #else
+static gboolean
+on_button_expose (GtkWidget *widget,
+                  GdkEventExpose *event,
+                  TaskTitle *title)
+{
+  g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
+
+  TaskTitlePrivate *priv;
+  priv = title->priv;
+
+  if (priv->mouse_in_close_button)
+  {
+    GtkStyle *style = gtk_widget_get_style (widget);
+
+    gtk_paint_box (style,
+                   event->window,
+                   GTK_STATE_PRELIGHT,
+                   GTK_SHADOW_NONE,
+                   NULL,
+                   NULL,
+                   NULL,
                    event->area.x,
                    event->area.y + 2,
                    event->area.width,
                    event->area.height - 4);
-#endif
   }
   return FALSE;
 }
+#endif
 
 static void
 on_name_changed (WnckWindow *window, TaskTitle *title)
@@ -297,8 +301,8 @@ on_active_window_changed (WnckScreen *screen,
   {
     gtk_label_set_text (GTK_LABEL (priv->label), 
                         wnck_window_get_name (act_window));
-    gtk_image_set_from_stock (GTK_IMAGE (priv->button_image), 
-                              GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->button_image),
+                                  "window-close", GTK_ICON_SIZE_MENU);
 
     gtk_widget_set_tooltip_text (GTK_WIDGET (title),
                                  wnck_window_get_name (act_window));
@@ -386,49 +390,42 @@ on_button_release (GtkWidget *title, GdkEventButton *event)
 }
 
 
+#if GTK_CHECK_VERSION (3, 0, 0)
 static gboolean
-#if GTK_CHECK_VERSION (3, 0, 0)
-on_draw (GtkWidget *eb, cairo_t *cr)
-#else
-on_expose (GtkWidget *eb, GdkEventExpose *event)
-#endif
+on_draw (GtkWidget *w, cairo_t *cr)
 {
-  GtkAllocation *allocation;
-  GtkStyle *style;
-  GtkStateType state;
+  if (gtk_widget_get_state_flags (w) == GTK_STATE_FLAG_ACTIVE) {
+    GtkStyleContext *context = gtk_widget_get_style_context (w);
+    gtk_render_frame (context,
+                      cr,
+                      0, 0,
+                      gtk_widget_get_allocated_width (w),
+                      gtk_widget_get_allocated_height (w));
+  }
 
-  gtk_widget_get_allocation (eb, allocation);
-  style = gtk_widget_get_style (eb);
-  state = gtk_widget_get_state (eb);
-
-  if (state == GTK_STATE_ACTIVE && allocation)
-    gtk_paint_box (style,
-#if GTK_CHECK_VERSION (3, 0, 0)
-                 cr,
-#else
-                 eb->window,
-#endif
-                 state, GTK_SHADOW_NONE,
-#if !GTK_CHECK_VERSION (3, 0, 0)
-                 NULL,
-#endif
-                 eb, "button",
-                 allocation->x, allocation->y, 
-                 allocation->width, allocation->height);
-  
-  
-#if GTK_CHECK_VERSION (3, 0, 0)
-  gtk_container_propagate_draw (GTK_CONTAINER (eb),
-                                gtk_bin_get_child (GTK_BIN (eb)),
+  gtk_container_propagate_draw (GTK_CONTAINER (w),
+                                gtk_bin_get_child (GTK_BIN (w)),
                                 cr);
-
-#else
-  gtk_container_propagate_expose (GTK_CONTAINER (eb), 
-                                  gtk_bin_get_child (GTK_BIN (eb)),
-                                  event);
-#endif
   return TRUE;
 }
+#else
+static gboolean
+on_expose (GtkWidget *w, GdkEventExpose *event)
+{
+  if (w->state == GTK_STATE_ACTIVE) {
+    gtk_paint_box (w->style, w->window,
+                   w->state, GTK_SHADOW_NONE,
+                   NULL, w, "button",
+                   w->allocation.x, w->allocation.y,
+                   w->allocation.width, w->allocation.height);
+  }
+
+  gtk_container_propagate_expose (GTK_CONTAINER (w),
+                                  gtk_bin_get_child (GTK_BIN (w)),
+                                  event);
+  return TRUE;
+}
+#endif
 
 /* GObject stuff */
 static void
@@ -466,7 +463,6 @@ task_title_init (TaskTitle *title)
   TaskTitlePrivate *priv;
   GdkScreen *gdkscreen;
   GtkIconTheme *theme;
-  GtkSettings *settings;
   GdkPixbuf *pixbuf;
   AtkObject *atk;
   int width, height;
@@ -487,14 +483,23 @@ task_title_init (TaskTitle *title)
                              0, 0, 6, 6);
   gtk_container_add (GTK_CONTAINER (title), priv->align);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  priv->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+#else
   priv->box = gtk_hbox_new (FALSE, 2);
+#endif
   gtk_container_add (GTK_CONTAINER (priv->align), priv->box);
   gtk_widget_set_no_show_all (priv->box, TRUE);
   gtk_widget_show (priv->box);
 
   priv->label = gtk_label_new (_("Home"));
   gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gtk_widget_set_halign (priv->label, GTK_ALIGN_START);
+  gtk_widget_set_valign (priv->label, GTK_ALIGN_CENTER);
+#else
   gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
+#endif
   
   PangoAttrList *attr_list = pango_attr_list_new ();
   PangoAttribute *attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
@@ -530,24 +535,12 @@ task_title_init (TaskTitle *title)
                     G_CALLBACK (on_button_expose), title);
 #endif
 
-  /* Load the quit icon.  We have to do this in such a god-forsaken way
-     because of http://bugzilla.gnome.org/show_bug.cgi?id=581359 and the
-     fact that we support as far back as GTK+ 2.12 (which never passes
-     FORCE_SIZE in GtkImage).  The only way to guarantee icon size is to
-     load and scale it ourselves.  We don't do this for all the other icons
-     in the source just because we know that this icon doesn't come in the
-     size we want. */
   gdkscreen = gtk_widget_get_screen (GTK_WIDGET (title));
   theme = gtk_icon_theme_get_for_screen (gdkscreen);
-  settings = gtk_settings_get_for_screen (gdkscreen);
-  if (!gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU,
-&width, &height))
-      width = height = 16;
-  width = MIN(width, height);
-  pixbuf = gtk_icon_theme_load_icon (theme, "mate-logout", width, 0, NULL);
-  priv->quit_icon = gdk_pixbuf_scale_simple (pixbuf, width, width,
-                                             GDK_INTERP_BILINEAR);
-  g_object_unref (G_OBJECT (pixbuf));
+  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU,
+                        &width, &height);
+
+  priv->quit_icon = gtk_icon_theme_load_icon (theme, "mate-logout", width, 0, NULL);
 
   priv->button_image = gtk_image_new_from_pixbuf (priv->quit_icon);
   gtk_container_add (GTK_CONTAINER (priv->button), priv->button_image);
